from invidious import *

searched = search("distrotube")

channels = []

for item in searched:
    if type(item) == ChannelItem:
        channel = get_channel(item.authorId)
        channels.append(channel)

for channel in channels:
    print(channel.author)




class ChannelItem():
    """ChannelItem class. Used in search."""

    def __init__(self, author="", authorId="", authorUrl="", authorThumbnails=[],
                       subCount=0, videoCount=0, description="", descriptionHtml=""):
        self.author = author
        self.authorId = authorId
        self.authorUrl = authorUrl
        self.authorThumbnails = authorThumbnails
        self.subCount = subCount
        self.videoCount = videoCount
        self.description = description
        self.descriptionHtml = descriptionHtml

    def loadFromDict(self, dct):
        """Loads class info from dictionary"""
        self.author = dct['author']
        self.authorId = dct['authorId']
        self.authorUrl = dct['authorUrl']
        self.authorThumbnails = dct['authorThumbnails']
        self.subCount = dct['subCount']
        self.videoCount = dct['videoCount']
        self.description = dct['description']
        self.descriptionHtml = dct['descriptionHtml']




from .playlist_item import PlaylistItem
from .channel_item import ChannelItem
from .recommended_video import RecommendedVideo
from .short_video import ShortVideo
from .video_item import VideoItem
from .channel import Channel
from .video import Video
import requests

HEADERS = ({
        "User-Agent":
        "Mozilla/5.0 (Windows NT 10.0; Win64; x64) "
        "AppleWebKit/537.36 (KHTML, like Gecko) Chrome"
        "/75.0.3770.100 Safari/537.36"
    })

def request(url: str):
    response = requests.get(url, headers=HEADERS)
    if response.status_code == 200: return response.json()

def search(query: str, page=0, sort_by="", duration="", date="", ctype="", feauters=[], region=""):
    """
    Invidious search method. Return list with VideoItem, ChannelItem, PlaylistItem.

    query: your search query.
    page: number of page.
    sort_by: [relevance, rating, upload_date, view_count].
    date: [hour, today, week, month, year].
    duration: [short, long].
    ctype: [video, playlist, channel, all] (default: video).
    feauters: [hd, subtitles, creative_commons, 3d, live, purchased, 4k, 360, location, hdr].
    region: ISO 3166 country code (default: US).
    """
    req = f"https://invidious.snopyta.org/api/v1/search?q={query}"
    if page > 0: req += f"&page={page}"
    if sort_by != "": req += f"&sort_by={sort_by}"
    if duration != "": req += f"&duration={duration}"
    if date != "": req += f"&date={date}"
    if ctype != "": req += f"&type={ctype}"
    if feauters != []:
        req += "&feauters="
        for feauter in feauters:
            req += feauter+","
        req = req[:len(req)-2]
    if region != "": req += f"region={region}"

    jdict = request(req)
    itemsList = []

    for item in jdict:
        citem = None
        if item['type'] == 'channel': citem = ChannelItem()
        elif item['type'] == 'video': citem = VideoItem()
        elif item['type'] == 'playlist': citem = PlaylistItem()
        if citem != None: 
            citem.loadFromDict(item)
            itemsList.append(citem)
    
    return itemsList

def get_video(videoId: str, region=""):
    """
    Invidious get_video method. Return Video by id.
    
    videoId: id of video.
    region: ISO 3166 country code (default: US).
    """
    req = f"https://invidious.snopyta.org/api/v1/videos/{videoId}"
    if region != "": req += f"&region={region}"

    response = request(req)
    if response == None:
        print(f"ERROR: Video with {videoId} id doesn't exists.")
        return None
        
    rVideos = response['recommendedVideos']
    recommendedVideos = []
    for item in rVideos:
        vitem = RecommendedVideo()
        vitem.loadFromDict(item)
        recommendedVideos.append(vitem)
    response['recommendedVideos'] = recommendedVideos

    video = Video()
    video.loadFromDict(response)

    return video

def get_channel(authorId: str, sort_by=""):
    """
    Invidious get_channel method. Return Channel by id.
    
    authorId: id of channel.
    sort_by: sorting channel videos. [newest, oldest, popular] (default: newest).
    """
    req = f"https://invidious.snopyta.org/api/v1/channels/{authorId}"
    if sort_by != "": req += f"&sort_by={sort_by}"

    response = request(req)
    if response == None: 
        print(f"ERROR: Channel with {authorId} id doesn't exists.")
        return None

    lVideos = response['latestVideos']
    latestVideos = []
    for item in lVideos:
        vitem = VideoItem()
        vitem.loadFromDict(item)
        latestVideos.append(vitem)
    response['latestVideos'] = latestVideos

    channel = Channel()
    channel.loadFromDict(response)

    return channel

def get_popular():
    """
    Invidious get_popular method. Return list with ShortVideo.
    """
    req = f"https://invidious.snopyta.org/api/v1/popular"

    response = request(req)
    if response == None: return None
        
    pVideos = response
    popularVideos = []
    for item in pVideos:
        vitem = ShortVideo()
        vitem.loadFromDict(item)
        popularVideos.append(vitem)

    return popularVideos

if __name__ == "__main__":
    print(get_channel("UCrPw1-nDPppBuh7UCBqu3GQ").author)
    print(get_video("L8Ohygt5UVY").title)
    items = search(query="kuplinov", date="today")
    for item in items:
        if type(item) == VideoItem:
            print(item.title)





class PlaylistItem():
    """PlaylistItem class. Used in search."""

    def __init__(self, title="", playlistId="", author="",
                       authorId="", authorUrl="", videoCount=0, videos=[]):
        """PlaylistItem class. Used in search."""
        self.title = title
        self.playlistId = playlistId
        self.author = author
        self.authorId = authorId
        self.authorUrl = authorUrl
        self.videoCount = videoCount
        self.videos = videos

    def loadFromDict(self, dct):
        """Loads class info from dictionary"""
        self.title = dct['title']
        self.playlistId = dct['playlistId']
        self.author = dct['author']
        self.authorId = dct['authorId']
        self.authorUrl = dct['authorUrl']
        self.videoCount = dct['videoCount']
        self.videos = dct['videos']









class Channel():
    """Base Channel class. Have maximum options."""

    def __init__(self, author="", authorId="", authorUrl="", authorBanners=[], authorThumbnails=[], 
                       subCount=0, totalViews=0, joined=0, autoGenerated=False, isFamilyFriendly=False,
                       description="", descriptionHtml="", allowedRegions=[],
                       latestVideos=[], relatedChannels=[]):
        self.author = author
        self.authorId = authorId
        self.authorUrl = authorUrl
        self.authorBanners = authorBanners
        self.authorThumbnails = authorThumbnails
        self.subCount = subCount
        self.totalViews = totalViews
        self.joined = joined
        self.autoGenerated = autoGenerated
        self.isFamilyFriendly = isFamilyFriendly
        self.description = description
        self.descriptionHtml = descriptionHtml
        self.allowedRegions = allowedRegions
        self.latestVideos = latestVideos
        self.relatedChannels = relatedChannels

    def loadFromDict(self, dct):
        """Loads class info from dictionary"""
        self.author = dct['author']
        self.authorId = dct['authorId']
        self.authorUrl = dct['authorUrl']
        self.authorBanners = dct['authorBanners']
        self.authorThumbnails = dct['authorThumbnails']
        self.subCount = dct['subCount']
        self.totalViews = dct['totalViews']
        self.joined = dct['joined']
        self.autoGenerated = dct['autoGenerated']
        self.isFamilyFriendly = dct['isFamilyFriendly']
        self.description = dct['description']
        self.descriptionHtml = dct['descriptionHtml']
        self.allowedRegions = dct['allowedRegions']
        self.latestVideos = dct['latestVideos']
        self.relatedChannels = dct['relatedChannels']









class VideoItem():
    """VideoItem class. Used in search."""

    def __init__(self, title="", videoId="", author="", authorId="", authorUrl="",
                       videoThumbnails=[], description="", descriptionHtml="",
                       viewCount=0, published=0, publishedText="", lengthSeconds=0, 
                       liveNow=False, premium=False, isUpcoming=False):
        """VideoItem class. Used in search."""
        self.title = title
        self.videoId = videoId
        self.author = author
        self.authorId = authorId
        self.authorUrl = authorUrl
        self.videoThumbnails = videoThumbnails
        self.description = description
        self.descriptionHtml = descriptionHtml
        self.viewCount = viewCount
        self.published = published
        self.publishedText = publishedText
        self.lengthSeconds = lengthSeconds
        self.liveNow = liveNow
        self.premium = premium
        self.isUpcoming = isUpcoming

    def loadFromDict(self, dct):
        """Loads class info from dictionary"""
        self.title = dct['title']
        self.videoId = dct['videoId']
        self.author = dct['author']
        self.authorId = dct['authorId']
        self.authorUrl = dct['authorUrl']
        self.videoThumbnails = dct['videoThumbnails']
        self.description = dct['description']
        self.descriptionHtml = dct['descriptionHtml']
        self.viewCount = dct['viewCount']
        self.published = dct['published']
        self.publishedText = dct['publishedText']
        self.lengthSeconds = dct['lengthSeconds']
        self.liveNow = dct['liveNow']
        self.premium = dct['premium']
        self.isUpcoming = dct['isUpcoming']









class ShortVideo():
    """ShortVideo class. Used in popular videos."""

    def __init__(self, title="", videoId=0, videoThumbnails=[], author="",
                 auhtorId="", auhtorUrl="", lengthSeconds=0, 
                 viewCount=0, published=0, publishedText=""):
        self.title = title
        self.videoId = videoId
        self.videoThumbnails = videoThumbnails
        self.author = author
        self.authorId = auhtorId
        self.authorUrl = auhtorUrl
        self.lengthSeconds = lengthSeconds
        self.viewCount = viewCount
        self.published = published
        self.publishedText = publishedText

    def loadFromDict(self, dct):     
        """Loads class info from dictionary"""   
        self.title = dct['title']
        self.videoId = dct['videoId']
        self.videoThumbnails = dct['videoThumbnails']
        self.author = dct['author']
        self.authorId = dct['authorId']
        self.authorUrl = dct['authorUrl']
        self.lengthSeconds = dct['lengthSeconds']
        self.viewCount = dct['viewCount']
        self.published = dct['published']
        self.publishedText = dct['publishedText']


from setuptools import setup, find_packages
import os

VERSION = '0.0.10'
DESCRIPTION = 'A Python wrapper for Invidious API'

with open("README.md", "r") as ofile:
    LONG_DESCRIPTION = ofile.read()

packages = [
    'invidious',
]

# Setting up
setup(
    name="invidious.py",
    version=VERSION,
    author="loliconshik3",
    author_email="loliconshik3@gmail.com",
    description=DESCRIPTION,
    long_description_content_type="text/markdown",
    long_description=LONG_DESCRIPTION,
    packages=packages,
    install_requires=['requests'],
    keywords=['python', 'invidious', 'youtube', 'video', 'api'],
    classifiers=[
        "Development Status :: 1 - Planning",
        "Intended Audience :: Developers",
        "Programming Language :: Python :: 3",
        "Operating System :: Unix",
        "Operating System :: MacOS :: MacOS X",
        "Operating System :: Microsoft :: Windows"
    ]
)



class Video():
    """Base Video class. Have maximum options."""

    def __init__(self, title="", videoId="", videoThumbnails=[], storyboards=[], description="", descriptionHtml="",
                       published=0, publishedText="", keywords=[], viewCount=0, likeCount=0, dislikeCount=0,
                       paid=False, premium=False, isFamilyFriendly=False, allowedRegions=[], genre="", genreUrl="",
                       author="", authorId="", authorUrl="", authorThumbnails=[], subCountText="", lengthSeconds=0,
                       allowRatings=False, rating=0, isListed=False, liveNow=False, isUpcoming=False, dashUrl="",
                       adaptiveFormats=[], formatStreams=[], captions=[], recommendedVideos=[]):
        self.title = title
        self.videoId = videoId
        self.videoThumbnails = videoThumbnails
        self.storyboards = storyboards
        self.description = description
        self.descriptionHtml = descriptionHtml
        self.published = published
        self.publishedText = publishedText
        self.keywords = keywords
        self.viewCount = viewCount
        self.likeCount = likeCount
        self.dislikeCount = dislikeCount
        self.paid = paid
        self.premium = premium
        self.isFamilyFriendly = isFamilyFriendly
        self.allowedRegions = allowedRegions
        self.genre = genre
        self.genreUrl = genreUrl
        self.author = author
        self.authorId = authorId
        self.authorUrl = authorUrl
        self.authorThumbnails = authorThumbnails
        self.subCountText = subCountText
        self.lengthSeconds = lengthSeconds
        self.allowRatings = allowRatings
        self.rating = rating
        self.isListed = isListed
        self.liveNow = liveNow
        self.isUpcoming = isUpcoming
        self.dashUrl = dashUrl
        self.adaptiveFormats = adaptiveFormats
        self.formatStreams = formatStreams
        self.captions = captions
        self.recommendedVideos = recommendedVideos

    def loadFromDict(self, dct):
        """Loads ChannelItem info from dictionary"""
        self.title = dct['title']
        self.videoId = dct['videoId']
        self.videoThumbnails = dct['videoThumbnails']
        self.storyboards = dct['storyboards']
        self.description = dct['description']
        self.descriptionHtml = dct['descriptionHtml']
        self.published = dct['published']
        self.publishedText = dct['publishedText']
        self.keywords = dct['keywords']
        self.viewCount = dct['viewCount']
        self.likeCount = dct['likeCount']
        self.dislikeCount = dct['dislikeCount']
        self.paid = dct['paid']
        self.premium = dct['premium']
        self.isFamilyFriendly = dct['isFamilyFriendly']
        self.allowedRegions = dct['allowedRegions']
        self.genre = dct['genre']
        self.genreUrl = dct['genreUrl']
        self.author = dct['author']
        self.authorId = dct['authorId']
        self.authorUrl = dct['authorUrl']
        self.authorThumbnails = dct['authorThumbnails']
        self.subCountText = dct['subCountText']
        self.lengthSeconds = dct['lengthSeconds']
        self.allowRatings = dct['allowRatings']
        self.rating = dct['rating']
        self.isListed = dct['isListed']
        self.liveNow = dct['liveNow']
        self.isUpcoming = dct['isUpcoming']
        self.dashUrl = dct['dashUrl']
        self.adaptiveFormats = dct['adaptiveFormats']
        self.formatStreams = dct['formatStreams']
        self.captions = dct['captions']
        self.recommendedVideos = dct['recommendedVideos']





from .channel_item import *
from .channel import *
from .playlist_item import *
from .video_item import *
from .video import *
from .recommended_video import *
from .short_video import *
from .invidious import *